@@ -43,6 +43,7 @@
 #include <libgnomeprint/gp-truetype-utils.h>
 #include <libgnomeprint/gp-tt-t1.h>
 #include <libgnomeprint/gnome-font-family.h>
+#include <libgnomeprint/gnome-print-encode.h>
 
 enum {
 	PROP_0,
@@ -70,6 +71,7 @@ static void gnome_font_face_get_prop (GObject *o, guint id, GValue *value, GPara
 static void gff_load_metrics (GnomeFontFace *face, gint glyph);
 static void gff_load_outline (GnomeFontFace *face, gint glyph);
 
+static void gf_pso_print_sized (GnomeFontPsObject *pso, const guchar *text, gint size);
 static void gf_pso_sprintf (GnomeFontPsObject * pso, const gchar * format, ...);
 static void gnome_font_face_ps_embed_ensure_size (GnomeFontPsObject * pso, gint size);
 
@@ -179,6 +181,11 @@ gnome_font_face_finalize (GObject * object)
 		face->ft_face = NULL;
 	}
 
+	if (face->psname) {
+		g_free (face->psname);
+		face->psname = NULL;
+	}
+
 	/* Fonts should reference us */
 	g_assert (face->fonts == NULL);
 
@@ -196,7 +203,7 @@ gnome_font_face_get_prop (GObject *o, guint id, GValue *value, GParamSpec *pspec
 
 	switch (id) {
 	case PROP_FONTNAME:
-		g_value_set_string (value, face->entry->psname);
+		g_value_set_string (value, face->psname);
 		break;
 	case PROP_FULLNAME:
 		g_value_set_string (value, face->entry->name);
@@ -327,7 +334,7 @@ gnome_font_face_get_ps_name (const GnomeFontFace * face)
 	g_return_val_if_fail (face != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), NULL);
 
-	return face->entry->psname;
+	return face->psname;
 }
 
 /**
@@ -766,6 +773,7 @@ gff_load (GnomeFontFace *face)
 	FT_CharMap charmap;
 	FT_CharMap appleRoman = 0;
 	FT_CharMap symbol = 0;
+	const guchar *psname;
 	int n;
 
 	if (!ft_library) {
@@ -782,6 +790,14 @@ gff_load (GnomeFontFace *face)
 	ft_result = FT_New_Face (ft_library, entry->file, 0, &ft_face);
 	g_return_val_if_fail (ft_result == FT_Err_Ok, FALSE);
 
+	psname = FT_Get_Postscript_Name (ft_face);
+	if (psname == NULL) {
+		g_warning ("PS name is NULL, for \"%s\" using fallback", entry->file);
+		face->psname = g_strdup ("Helvetica");
+	} else {
+		face->psname = g_strdup (psname);
+	}
+	
 	/* FIXME: scalability (Lauris) */
 	/* FIXME: glyph names (Lauris) */
 
@@ -877,7 +893,9 @@ gff_load_outline (GnomeFontFace *face, gint glyph)
 	g_assert (face->ft_face);
 	g_assert (!face->glyphs[glyph].bpath);
 
-	FT_Load_Glyph (face->ft_face, glyph, FT_LOAD_NO_SCALE | FT_LOAD_NO_HINTING | FT_LOAD_NO_BITMAP);
+	FT_Load_Glyph (face->ft_face, glyph,
+		       FT_LOAD_NO_SCALE | FT_LOAD_NO_HINTING |
+		       FT_LOAD_NO_BITMAP | FT_LOAD_IGNORE_TRANSFORM);
 
 	a[0] = a[3] = face->ft2ps;
 	a[1] = a[2] = a[4] = a[5] = 0.0;
@@ -1027,7 +1045,7 @@ static void gnome_font_face_ps_embed_ensure_size (GnomeFontPsObject *pso, gint s
  */
 
 GnomeFontPsObject *
-gnome_font_face_pso_new (GnomeFontFace *face, guchar *residentname)
+gnome_font_face_pso_new (GnomeFontFace *face, guchar *residentname, gint instance)
 {
 	GnomeFontPsObject *pso;
 
@@ -1040,7 +1058,10 @@ gnome_font_face_pso_new (GnomeFontFace *face, guchar *residentname)
 	g_object_ref (G_OBJECT (face));
 	if (residentname)
 		pso->residentname = g_strdup (residentname);
-	pso->encodedname = g_strdup_printf ("GnomeUni-%s", face->entry->psname);
+	if (instance == 0)
+		pso->encodedname = g_strdup_printf ("GnomeUni-%s", face->psname);
+	else
+		pso->encodedname = g_strdup_printf ("GnomeUni-%s_%03d", face->psname, instance);
 	pso->bufsize = 0;
 	pso->length = 0;
 	pso->buf = NULL;
@@ -1125,7 +1146,7 @@ gnome_font_face_ps_embed_t1 (GnomeFontPsObject *pso)
 
 	g_return_if_fail (pso->face->entry->type == GP_FONT_ENTRY_TYPE1);
 	file_name = pso->face->entry->file;	
-	embeddedname = pso->face->entry->psname;
+	embeddedname = pso->face->psname;
 
 	if (!GFF_LOADED (pso->face)) {
 		g_warning ("file %s: line %d: Face: %s: Cannot load face", __FILE__, __LINE__, pso->face->entry->name);
@@ -1276,6 +1297,8 @@ gnome_font_face_ps_embed_t1 (GnomeFontPsObject *pso)
  * Adobe Inc., The Type42 Font Format Specification, 1998 <http://partners.adobe.com/asn/developer/PDFS/TN/5012.Type42_Spec.pdf>
  */
 
+#define TT_BLOCK_SIZE 1024
+
 static void
 gnome_font_face_ps_embed_tt (GnomeFontPsObject *pso)
 {
@@ -1287,6 +1310,8 @@ gnome_font_face_ps_embed_tt (GnomeFontPsObject *pso)
 	gdouble TTVersion, MfrRevision;
 	const ArtDRect *bbox;
 	gint i;
+	gchar *line;
+	gint line_length;
 
 	g_return_if_fail (pso->face->entry->type == GP_FONT_ENTRY_TRUETYPE);
  	file_name = pso->face->entry->file;
@@ -1296,7 +1321,7 @@ gnome_font_face_ps_embed_tt (GnomeFontPsObject *pso)
 		return;
 	}
 	
-	embeddedname = pso->face->entry->psname;
+	embeddedname = pso->face->psname;
 
 	fbuf = b.buf;
 	strings = gp_tt_split_file (fbuf, b.buf_size);
@@ -1320,23 +1345,27 @@ gnome_font_face_ps_embed_tt (GnomeFontPsObject *pso)
 		/* fixme: XUID */
 		/* fixme: Be more intelligent */
 		gf_pso_sprintf (pso, "/sfnts [\n");
+		line_length = gnome_print_encode_hex_wcs (TT_BLOCK_SIZE);
+		line = g_malloc (line_length);
 		while (strings) {
-			guint start, next, size, i;
+			guint start, next;
+			guchar *s, *e;
 			start = GPOINTER_TO_UINT (strings->data);
 			strings = g_slist_remove (strings, strings->data);
 			next = strings ? GPOINTER_TO_UINT (strings->data) : b.buf_size;
-			size = next - start;
 			gf_pso_sprintf (pso, "<\n");
-			for (i = start; i < next; i+= 32) {
-				gint e, j;
-				e = MIN (i + 32, next);
-				for (j = i; j < e; j++) {
-					gf_pso_sprintf (pso, "%.2x", *(fbuf + j));
-				}
-				gf_pso_sprintf (pso, "\n");
+			s = b.buf + start;
+			e = b.buf + next;
+			while (s < e) {
+				gint block_size, len;
+				block_size = MIN (TT_BLOCK_SIZE, e - s);
+				len = gnome_print_encode_hex (s, line, block_size);
+				gf_pso_print_sized (pso, line, len);
+				s += block_size;
 			}
 			gf_pso_sprintf (pso, strings ? ">\n" : "00>\n");
 		}
+		g_free (line);
 		gf_pso_sprintf (pso, "] def\n");
 		/* fixme: Use CID or something */
 		gf_pso_sprintf (pso, "/CharStrings %d dict dup begin\n", pso->face->num_glyphs);
@@ -1479,7 +1508,15 @@ gnome_font_face_ps_embed_empty (GnomeFontPsObject *pso)
 }
 
 static void
-gf_pso_sprintf (GnomeFontPsObject * pso, const gchar * format, ...)
+gf_pso_print_sized (GnomeFontPsObject *pso, const guchar *text, gint size)
+{
+	gnome_font_face_ps_embed_ensure_size (pso, size);
+	memcpy (pso->buf + pso->length, text, size);
+	pso->length += size;
+}
+
+static void
+gf_pso_sprintf (GnomeFontPsObject *pso, const gchar * format, ...)
 {
 	va_list arguments;
 	gchar *oldlocale;
@@ -1494,9 +1531,7 @@ gf_pso_sprintf (GnomeFontPsObject * pso, const gchar * format, ...)
 	va_end (arguments);
 
 	len = strlen (text);
-	gnome_font_face_ps_embed_ensure_size (pso, len);
-	memcpy (pso->buf + pso->length, text, len);
-	pso->length += len;
+	gf_pso_print_sized (pso, text, len);
 	g_free (text);
 
 	setlocale (LC_NUMERIC, oldlocale);
