@@ -1,8 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-
-#ifndef __GPA_KEY_H__
-#define __GPA_KEY_H__
-
 /*
  *  gpa-key.h: 
  *
@@ -24,59 +20,46 @@
  *    Lauris Kaplinski <lauris@ximian.com>
  *    Jose M. Celorio <chema@ximian.com>
  *
- *  Copyright (C) 2000-2001 Ximian, Inc. and Jose M. Celorio
+ *  Copyright (C) 2000-2003 Ximian, Inc.
  *
  */
 
+#ifndef __GPA_KEY_H__
+#define __GPA_KEY_H__
+
 #include <glib.h>
+#include <libxml/tree.h>
+#include "gpa-node.h"
 
 G_BEGIN_DECLS
 
 #define GPA_TYPE_KEY         (gpa_key_get_type ())
 #define GPA_KEY(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), GPA_TYPE_KEY, GPAKey))
-#define GPA_KEY_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST ((k),    GPA_TYPE_KEY, GPAKeyClass))
 #define GPA_IS_KEY(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), GPA_TYPE_KEY))
-#define GPA_IS_KEY_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k),    GPA_TYPE_KEY))
-#define GPA_KEY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o),  GPA_TYPE_KEY, GPAKeyClass))
+
+#define GPA_KEY_ID(k)         (GPA_KEY (k)->id)
+#define GPA_KEY_HAS_OPTION(k) ((k) && GPA_KEY (k)->option)
+#define GPA_KEY_OPTION(k)     ((k) && GPA_KEY (k)->option ? GPA_OPTION (GPA_KEY (k)->option) : NULL)
 
 typedef struct _GPAKey      GPAKey;
 typedef struct _GPAKeyClass GPAKeyClass;
 
-#include <libxml/tree.h>
-#include "gpa-list.h"
-
-/* GPAKey */
-
-#define GPA_KEY_VALUE_IS_QUARK_SHIFT (GPA_MODIFIED_LAST_SHIFT + 1)
-#define GPA_KEY_VALUE_IS_QUARK_FLAG (1 << GPA_KEY_VALUE_IS_QUARK_SHIFT)
-#define GPA_KEY_LAST_SHIFT GPA_KEY_VALUE_IS_QUARK_SHIFT
-
-#define GPA_KEY_ID(k) (GPA_KEY (k)->id)
-#define GPA_KEY_HAS_OPTION(k) ((k) && GPA_KEY (k)->option)
-#define GPA_KEY_OPTION(k) ((k) && GPA_KEY (k)->option ? GPA_OPTION (GPA_KEY (k)->option) : NULL)
-
 struct _GPAKey {
 	GPANode node;
 
-	GPANode *children;
 	GPANode *option;
 	guchar *value;
 };
 
-
 struct _GPAKeyClass {
 	GPANodeClass node_class;
 };
 
 GType gpa_key_get_type (void);
 
-GPANode *  gpa_key_new_from_option (GPANode *node);
-gboolean   gpa_key_copy            (GPANode *dst, GPANode *src);
 gboolean   gpa_key_merge_from_tree (GPANode *key, xmlNodePtr tree);
 gboolean   gpa_key_merge_from_key  (GPAKey *dst, GPAKey *src);
-xmlNodePtr gpa_key_write (xmlDocPtr doc, GPANode *key);
-
-
+xmlNodePtr gpa_key_to_tree (GPAKey *key);
 
 G_END_DECLS
 
