@@ -483,7 +483,7 @@ gpa_printer_get_default (void)
 		return NULL;
 	}
 
-	return gpa_list_get_defalt (printers_list);
+	return gpa_list_get_default (printers_list);
 }
 
 /**
@@ -534,7 +534,7 @@ gpa_printer_get_default_settings (GPAPrinter *printer)
 	g_return_val_if_fail (printer != NULL, NULL);
 	g_return_val_if_fail (GPA_IS_PRINTER (printer), NULL);
 
-	child = gpa_list_get_defalt (GPA_LIST (printer->settings));
+	child = gpa_list_get_default (GPA_LIST (printer->settings));
 
 	return child;
 }
@@ -565,7 +565,7 @@ gpa_printer_new (const gchar *id, const gchar *name, GPAModel *model, GPASetting
 	g_return_val_if_fail (GPA_IS_MODEL (model), NULL);
 	g_return_val_if_fail (settings != NULL, NULL);
 	g_return_val_if_fail (GPA_IS_SETTINGS (settings), NULL);
-	g_return_val_if_fail (gpa_initialized, NULL);
+	g_return_val_if_fail (gpa_initialized (), NULL);
 
 	check = gpa_printer_get_by_id (id);
 	if (check) {
