@@ -65,6 +65,10 @@ xmlChar *model_xml_template =
 "          <Key Id=\"Bottom\" Value=\"0\"/>"
 "        </Key>"
 "      </Option>"
+"      <Option Id=\"Job\">"
+"        <Option Id=\"NumCopies\" Type=\"String\" Default=\"1\"/>"
+"        <Option Id=\"Collate\" Type=\"String\" Default=\"false\"/>"
+"      </Option>"
 "    </Option>"
 "    <Option Id=\"Document\">"
 "      <Option Id=\"Page\">"
@@ -138,6 +142,35 @@ get_paper_text (ppd_file_t *ppd, ppd_size_t *size)
 	return g_strdup (size->name);
 }
 
+static GPANode *
+option_list_new_with_default (GPANode      *parent,
+			      const         guchar *id,
+			      ppd_option_t *option)
+{
+	char *defchoice = g_strdup (option->defchoice);
+	char *p = defchoice + strlen (defchoice);
+	ppd_choice_t *choice;
+
+	/* Strip trailing spaces and tabs since CUPS doesn't does this
+	 */
+	while (p > defchoice &&
+	       (*(p - 1) == ' ' || *(p - 1) == '\t')) {
+		*(p - 1) = '\0';
+		p--;
+	}
+
+	choice = ppdFindChoice (option, defchoice);
+	g_free (defchoice);
+	
+	if (!choice && option->num_choices > 0)
+		choice = &option->choices[0];
+		
+	if (!choice)
+		return NULL;
+
+	return gpa_option_list_new (parent, id, choice->choice);
+}
+
 static GPANode *
 load_paper_sizes (ppd_file_t *ppd, GPANode *parent)
 {
@@ -153,8 +186,10 @@ load_paper_sizes (ppd_file_t *ppd, GPANode *parent)
 		 */
 		return NULL;
 	
-	node = gpa_option_list_new (parent, "PhysicalSize",
-				    option->defchoice);
+	node = option_list_new_with_default (parent, "PhysicalSize",
+					     option);
+	if (!node)
+		return NULL;
 			
 	for (i = 0; i < ppd->num_sizes; i++) {
 		GPANode *size;
@@ -192,8 +227,10 @@ load_paper_sources (ppd_file_t *ppd, GPANode *parent)
 	if (!option)
 		return;
 	
-	node = gpa_option_list_new (parent, "PaperSource",
-				    option->defchoice);
+	node = option_list_new_with_default (parent, "PaperSource",
+					     option);
+	if (!node)
+		return;
 	
 	for (i = 0; i < option->num_choices; i++)
 		gpa_option_item_new (node,
@@ -250,7 +287,8 @@ append_printer (GPAList *printers_list, const char *name, gboolean is_default)
 	filename = cupsGetPPD (name);
 	ppd = ppdOpenFile (filename);
 	if (ppd == NULL)
-		goto append_printer_exit;
+		/* See bug#: 102938 */
+		return FALSE;
 
 	model = get_model (name, ppd);
 	if (model == NULL)
@@ -277,7 +315,7 @@ append_printer (GPAList *printers_list, const char *name, gboolean is_default)
 
 		my_gpa_node_unref (printer);
 		my_gpa_node_unref (GPA_NODE (model));
-		my_gpa_node_unref (gpa_node_unref (settings));
+		my_gpa_node_unref (settings);
 	}
 
 	if (ppd) {
